@@ -52,11 +52,14 @@ def ifunc(obj):
 class BioName:
     def __init__(self, names: Union[list, pd.Series, tuple], style='scientificName'):
         self.names = names
-        self.querys = self.build_querys(self.names)
+        self.querys = {}
         self.cache = {'ipni': {}, 'col': {}, 'powo': {}}
         self.style = style
 
     def get(self, action, typ=list, mark=False):
+        if self.querys == {}:
+            self.querys = self.build_querys()
+        # results 只包含有检索有结果的
         results = self.__build_cache_and_get_results(action)
         if results:
             if typ is list:
@@ -73,7 +76,7 @@ class BioName:
         """ 对 get 的结果进行组装
 
         results: 从 self.cache 中解析得到的结果字典, 每个元素都为元组
-        mark: 如果为 True，没有获得结果的检索词会被虽结果返回并以!标记
+        mark: 如果为 True，没有获得结果的检索词会被当成结果返回并以!标记
               如果为 None，没有检索结果则返回由 None 组成的与其他结果等长的元组
 
         return: 按照 self.names 中的元素顺序排列的检索结果 list，list 中的元素
@@ -88,11 +91,11 @@ class BioName:
                     try:
                         results[name] = ["".join(["!", name])]
                     except TypeError:
-                        if name:
+                        if pd.isnull(name):
+                            results[name] = [None]
+                        else:
                             # 如果 name 不是 None, 说明检索词有错误
                             results[name] = ["".join(["!", str(name)])]
-                        else:
-                            results[name] = [None]
                     results[name].extend([None]*(result_len - 1))
             else:
                 if self.querys[name] is None or name not in results:
@@ -233,7 +236,11 @@ class BioName:
         self.sema = asyncio.Semaphore(500, loop=loop)
         tasks = self.build_tasks(get_action[action], search_terms)
         results = loop.run_until_complete(tasks)
-        loop.close()
+        # 修复 Windows 下出现的 RuntimeErro: Event loop is closed
+        # 为什么注销 close 会管用，我也没完全搞清楚
+        # 猜测可能是 asyncio 会自动关闭 loop
+        # 暂且先这样吧！
+        #loop.close()
         self.pbar.close()
         for res in results:
             try:
@@ -284,7 +291,11 @@ class BioName:
             # 如果没有名称处理，默认将本名称作为接受名返回
             # 有些名字可能属于 unsolved 状态，也会被作为接受名返回
             pass
-        return " ".join([query_result['name'], query_result['author']]),
+        try:
+            return " ".join([query_result['name'], query_result['author']]),
+        except KeyError:
+            # 对于原变种，返回的结果中没有 author 属性，先直接返回 name
+            return query_result['name'],
 
     def powo_name(self, query_result):
         scientific_name = query_result["name"]
@@ -469,7 +480,13 @@ class BioName:
                     t = []
                     for a in r["authorTeam"]:
                         t.append(a["name"])
-                    std_teams[n] = self.code_authors(t)
+                    if t:
+                        std_teams[n] = self.code_authors(t)
+                    else:
+                        # ipni 一些学名的检索返回会有 authorTeam = []
+                        # 但 authors 属性却有合法值的情况，此时可以基于 authors
+                        # 生成可用于比对的 authorTeam，然后再进行编码
+                        std_teams[n] = self.code_authors(self.get_author_team(r['authors']))
                 # 开始比对原命名人与可选学名的命名人的比对结果
                 scores = self.contrast_code(aut_codes, std_teams)
                 # print(scores)
@@ -648,81 +665,98 @@ class BioName:
         else:
             return filters.value['kew']
 
-    def format(self, p='scientificName'):
-        std_names = self.build_querys(self.names)
-        if p == 'simpleName':
-            return [
-                std_names[name][0]
-                if std_names[name] else None
-                for name in self.names
-            ]
-        elif p == 'scientificName':
-            return [
-                ' '.join([std_names[name][0], std_names[name][2]])
-                if std_names[name] else None
-                for name in self.names
-            ]
-
-    def build_querys(self, names):
-        raw2stdname = dict.fromkeys(names)
+    def build_querys(self):
+        raw2stdname = dict.fromkeys(self.names)
         for raw_name in raw2stdname:
-            raw2stdname[raw_name] = self.format_name(raw_name)
+            split_name = self._format_name(raw_name)
+            if split_name is None:
+                raw2stdname[raw_name] = None
+                continue
+            else:
+                if split_name[2] == "" and split_name[3]:
+                    simple_name = ' '.join([split_name[0], split_name[1], split_name[3]]).strip()
+                else:
+                    simple_name = ' '.join(split_name[:4]).strip()
+                raw2stdname[raw_name] = simple_name, split_name[-2], split_name[4], split_name[-1]
         return raw2stdname
 
-    def format_name(self, raw_name):
+    def format_latin_names(self, pattern):
+        raw2stdname = dict.fromkeys(self.names)
+        for raw_name in raw2stdname:
+            split_name = self._format_name(raw_name)
+            if split_name is None:
+                raw2stdname[raw_name] = None
+                continue
+            if pattern == 'simpleName':
+                if split_name[2] == "" and split_name[3]:
+                    simple_name = ' '.join([split_name[0], split_name[1], split_name[3]]).strip()
+                else:
+                    simple_name = ' '.join(split_name[:4]).strip()
+                raw2stdname[raw_name] = simple_name
+            elif pattern == 'scientificName':
+                if split_name[2] == "" and split_name[3]:
+                    simple_name = ' '.join([split_name[0], split_name[1], split_name[3]]).strip()
+                else:
+                    simple_name = ' '.join(split_name[:4]).strip()
+                author = split_name[4]
+                raw2stdname[raw_name] = ' '.join([simple_name, author]) if author else simple_name
+            elif pattern == 'plantSplitName':
+                raw2stdname[raw_name] = tuple(e if e !='' else None for e in split_name[:5])
+            elif pattern == 'fullPlantSplitName':
+                elements = (split_name[0], split_name[1], split_name[5], split_name[2], split_name[3], split_name[4])
+                raw2stdname[raw_name] = tuple(e if e !='' else None for e in elements)
+            elif pattern == 'animalSplitName':
+                elements = (split_name[0], split_name[1], split_name[3], split_name[4])
+                raw2stdname[raw_name] = tuple(e if e !='' else None for e in elements)
+            else:
+                raise ValueError("学名处理参数错误，不存在{}".format(pattern))
+        return [raw2stdname[name] for name in self.names]
+
+    def _format_name(self, raw_name):
         """ 将手写学名转成规范格式
 
-            raw_name: 各类动植物学名，目前仅支持:
+            raw_name: 各类动植物学名字符串，目前仅支持:
                       属名 x 种名 种命名人 种下加词 种下命名人
                       这类学名格式的清洗，
                       其中杂交符、种命名人、种下命名人均可缺省。
-            return: (raw_name 去命名人的学名, Filters定义的学名等级, 原始名称提取的命名人, raw_name)
-                    目前仍有个别命名人十分复杂的名称，无法准确提取命名人，使用时需注意
+            return: 命名人的各个组成部分构成的元组
                     如果无法提取合法的学名，则返回 None
         """
         species_pattern = re.compile(
-            r"\b([A-Z][a-zàäçéèêëöôùûüîï-]+)\s*([×Xx])?\s*([a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï-]+)?\s*(.*)")
+            r"(!?\b[A-Z][a-zàäçéèêëöôùûüîï-]+)\s*([×Xx])?\s*([a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï-]+)?\s*(.*)")
         subspecies_pattern = re.compile(
-            r"(.*?)\s*(var\.|subvar\.|subsp\.|ssp\.|f\.|fo\.|subf\.|form|cv\.|cultivar\.)?\s*([a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï-]+)\s*([\(（A-Z]*.*)")
+            r"([A-Z\(].*?[^A-Z])?\s*(var\.|subvar\.|subsp\.|ssp\.|f\.|fo\.|subf\.|form|cv\.|cultivar\.)?\s*\b([a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï][a-zàäçéèêëöôùûüîï-]+)\s*([（A-Z\(].*?[^A-Z])?$")
         try:
             species_split = species_pattern.findall(raw_name)[0]
         except BaseException:
             return None
         subspec_split = subspecies_pattern.findall(species_split[3])
 
+        genus = species_split[0]
+        if species_split[1] == "":
+            species = species_split[2]
+        else:
+            species = " ".join([species_split[1], species_split[2]])
         if subspec_split == []:
-            simple_name = " ".join([species_split[0], species_split[2]]) if \
-                species_split[1] == "" else " ".join(
-                [species_split[0], species_split[1], species_split[2]]
-            )
-            author = species_split[3]
+            authors = species_split[3]
+            taxon_rank = ""
+            infraspecies = ""
+            first_authors = ""
             if species_split[2] != "":
-                rank = Filters.specific
-            elif species_split[0].lower().endswith((
+                platform_rank = Filters.specific
+            elif genus.lower().endswith((
                 "aceae", "idae", "umbelliferae", "labiatae", "compositae", "gramineae",
                     "leguminosae")):
-                rank = Filters.familial
+                platform_rank = Filters.familial
             else:
-                rank = Filters.generic
+                platform_rank = Filters.generic
         else:
-            if subspec_split[0][1]:
-                simple_name = " ".join(
-                    [species_split[0], species_split[2],
-                        subspec_split[0][1], subspec_split[0][2]]
-                ) if species_split[1] == "" else " ".join(
-                    [species_split[0], species_split[1], species_split[2],
-                     subspec_split[0][1], subspec_split[0][2]]
-                )
-            else:
-                simple_name = " ".join(
-                    [species_split[0], species_split[2], subspec_split[0][2]]
-                ) if species_split[1] == "" else " ".join(
-                    [species_split[0], species_split[1],
-                     species_split[2], subspec_split[0][2]]
-                )
-            author = subspec_split[0][3]
-            rank = Filters.infraspecific
-        return simple_name.strip(), rank, author, raw_name
+            first_authors = subspec_split[0][0]
+            taxon_rank = subspec_split[0][1]
+            infraspecies = subspec_split[0][2]
+            authors = subspec_split[0][3]
+            platform_rank = Filters.infraspecific
+        return genus, species, taxon_rank, infraspecies, authors, first_authors, platform_rank, raw_name
 
     def contrast_code(self, raw_code, std_codes):
         """ 将一个学名的命名人和一组学名的命名人编码进行比较，以确定最匹配的
@@ -810,10 +844,12 @@ class BioName:
         return author_team
 
     def __call__(self, mark=True):
-        if input("是否要进行学名处理？（y/n）\n") == 'y':
-            pass
-        else:
-            return pd.DataFrame(self.names)
+        """
+        这里定义了对名称进行在线比对后，单纯返回学名，该采用哪种样式的逻辑
+        不管检索结果如何，程序都会返回与相应样式名称匹配的 DataFrame
+        对于检索结果中，有部分名称没有返回值，这些名称默认将使用"!"标注
+        对于检索结果中，所有的名称都没有返回值，则直接返回以对应列数 None 组成的 DataFrame
+        """
         if self.style == 'scientificName':
             choose = input(
                 "\n是否执行拼写检查，在线检查将基于 sp2000.org.cn、ipni.org 进行，但这要求工作电脑一直联网，同时如果需要核查的名称太多，可能会耗费较长时间（y/n）\n")
@@ -830,7 +866,7 @@ class BioName:
                 else:
                     return pd.DataFrame([None] * len(self.names))
             else:
-                return pd.DataFrame(self.format(p="scientificName"))
+                return pd.DataFrame(self.format_latin_names(pattern="scientificName"))
         elif self.style == 'simpleName':
             choose = input(
                 "\n是否执行拼写检查，在线检查将基于 sp2000.org.cn、ipni.org 进行，但这要求工作电脑一直联网，同时如果需要核查的名称太多，可能会耗费较长时间（y/n）\n")
@@ -847,7 +883,46 @@ class BioName:
                 else:
                     return pd.DataFrame([None]*len(self.names))
             else:
-                return pd.DataFrame(self.format(p="simpleName"))
+                return pd.DataFrame(self.format_latin_names(pattern="simpleName"))
+        elif self.style == 'plantSplitName':
+            choose = input(
+                "\n是否执行拼写检查，在线检查将基于 sp2000.org.cn、ipni.org 进行，但这要求工作电脑一直联网，同时如果需要核查的名称太多，可能会耗费较长时间（y/n）\n")
+            if choose.strip() == 'y':
+                results = self.get('stdName', mark=mark)
+                if results:
+                    return pd.DataFrame(
+                        [
+                            self._format_name(name[0].strip())[:4] + (name[1],)
+                            if name[1] else self._format_name(name[0].strip())[:5]
+                            if name[0] else (None, None, None, None, None)
+                            for name in results
+                        ]
+                    )
+                else:
+                    return pd.DataFrame([[None, None, None, None, None]] * len(self.names))
+            else:
+                results = self.format_latin_names(pattern="plantSplitName")
+                if set(results) == {None}:
+                    results = [[None, None, None, None, None]] * len(results)
+                return pd.DataFrame(results)
+        elif self.style == 'fullPlantSplitName':
+            # 该模式在这里刻意不提供在线名称比对
+            # 我们不是很赞成对植物学名进行包含种下命名人和种命名人的拆分
+            # 但进行要素全拆分，有的时候又是必须，
+            # 所以这里暂时只将其作为一个拆分功能，供用户进行学名拆分
+            results = self.format_latin_names(pattern="fullPlantSplitName")
+            if set(results) == {None}:
+                results = [[None, None, None, None, None, None]] * len(results)
+            return pd.DataFrame(results)
+        elif self.style == 'animalSplitName':
+            results = self.format_latin_names(pattern="animalSplitName")
+            if set(results) == {None}:
+                results = [[None, None, None, None]] * len(results)
+            return pd.DataFrame(results)
+        else:
+            print("\n学名处理参数有误，不存在{}\n".format(self.style))
+            return pd.DataFrame(self.names)
+
 
 
 @ifunc
@@ -1174,8 +1249,9 @@ class DateTime:
 
 @ifunc
 class HumanName:
-    def __init__(self, names: Union[list, pd.Series, tuple]):
+    def __init__(self, names: Union[list, pd.Series, tuple], separator='，'):
         self.names = names
+        self.separator = separator
 
     def format_names(self):
         """
@@ -1220,7 +1296,7 @@ class HumanName:
                             names[i] = name.replace(" ", "")
                     else:
                         names[i] = self.format_westname(name)
-                names_mapping[rec_names] = ", ".join(names)
+                names_mapping[rec_names] = self.separator.join(names)
             except NameError:
                 continue
             except BaseException:
@@ -1313,18 +1389,20 @@ class AdminDiv:
     def _build_mapping(self, raw_region, std_regions):
         score = (0, 0)  # 分别记录匹配的字数和匹配项的长度
         region = raw_region.rstrip(":")
+        region_index = len(region) - 1
         for stdregion in std_regions:
+            len_stdregion = len(stdregion)
             each_score = 0  # 记录累计匹配的字数
             i = 0  # 记录 region 参与匹配的起始字符位置
             j = 0  # 记录 stdregion 参与匹配的起始字符位置
-            while i < len(region)-1 and j < len(stdregion)-1:  # 单字无法匹配
+            while i < region_index and j < len_stdregion-1:  # 单字无法匹配
                 k = 2  # 用于初始化、递增可匹配的字符长度
                 n = stdregion[j:].find("::"+region[i:i+k])
                 m = 0  # 记录最终匹配的字符数
                 if n != -1:
                     # 白云城矿区可能会错误的匹配“中国,内蒙古自治区,白云鄂博矿区”
                     if region[-2] in stdregion:
-                        while n != -1 and k <= len(region[i:]):
+                        while n != -1 and k <= region_index - i + 1:
                             k += 1
                             m = n
                             n = stdregion[j:].find(region[i:i+k])
@@ -1342,11 +1420,11 @@ class AdminDiv:
                 if self.region_mapping[raw_region] in stdregion:
                     pass
                 # 优先匹配短的行政区，避免“河北”匹配“中国,天津,天津市,河北区“
-                elif len(stdregion) < score[1]:
-                    score = each_score, len(stdregion)
+                elif len_stdregion < score[1]:
+                    score = each_score, len_stdregion
                     self.region_mapping[raw_region] = "!"+stdregion
             elif each_score > score[0]:
-                score = each_score, len(stdregion)
+                score = each_score, len_stdregion
                 self.region_mapping[raw_region] = stdregion
         if score == (0, 0):
             if raw_region == "中国":
@@ -1766,23 +1844,23 @@ class UniqueID:
         self.duplicated = self.df.duplicated(keep=False)
         marks = [
             "".join(["!", m]) if d and not pd.isnull(m) else m
-            for m, d in zip(self.df[self.df.columns[0]], self.duplicated)
+            for m, d in zip(self.df.iloc[:, 0], self.duplicated)
         ]
         return marks
 
     def __call__(self):
-        self.df[self.df.columns[0]] = self.mark_duplicate()
-        return self.df
+        self.df.iloc[:, 0] = self.mark_duplicate()
+        return self.df.iloc[:, [0]]
 
 
 @ifunc
 class FillNa:
-    def __init__(self, df: Union[pd.DataFrame, pd.Series], fval):
+    def __init__(self, df: Union[pd.DataFrame, pd.Series], value=None, method=None, limit=None, downcast=None):
         self.df = df
-        self.fval = fval
+        self.fval = value
 
     def __call__(self):
-        return pd.DataFrame(self.df.fillna(self.fval))
+        return pd.DataFrame(self.df.fillna(self.fval, method=None, limit=None, downcast=None))
 
 
 @ifunc
