@@ -6,12 +6,13 @@ class CvhTerms(Enum):
     条形码 = UniqueID('$catalogNumber')
     馆代码 = RadioInput('$collectionCode', 'collectionCode')
     流水号 = UniqueID('$otherCatalogNumbers')
-    标本状态 = RadioInput('$lifeStage', 'lifeStage')
+    模式类型 = RadioInput('$typeStatus', 'typeStatus')
     库存 = RadioInput('$disposition', 'disposition')
+    标本状态 = RadioInput('$lifeStage', 'lifeStage')
     采集人 = HumanName('$recordedBy')
     采集号 = '$recordNumber'
-    份数 = Number('$individualCount', None, int)
     采集日期 = DateTime('$eventDate')
+    份数 = Number('$individualCount', None, int)
     国家 = '$country'
     province = '$province'
     city = '$city'
@@ -19,37 +20,29 @@ class CvhTerms(Enum):
     省市 = ('$province', '$city', ',')
     区县 = '$county'
     小地点 = ('$locality', '$mountain', '$waterBody', ',')
-    纬度__经度 = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
+    生境 = '$habitat'
+    科 = '$family'
+    拉丁名 = '$scientificName'
+    属__种__定名人__种下等级__种下加词__种下等级定名人 = BioName([('$genus', '$specificEpithet',  '$specificAuthorship', '$taxonRank', '$infraspecificEpithet', '$scientificNameAuthorship', ' '), '$拉丁名'], style='fullPlantSplitName')
+    中文名 = '$vernacularName'
+    鉴定人 = HumanName('$identifiedBy')
+    鉴定日期 = DateTime('$dateIdentified')
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
+    经度 = '$decimalLongitude'
+    纬度 = '$decimalLatitude'
     _海拔 = Number('$minimumElevationInMeters')
     _海拔高 = Number('$maximumElevationInMeters')
     海拔__海拔高 = Number('$minimumElevationInMeters', '$maximumElevationInMeters')
-    生境 = '$habitat'
-    中文名 = '$vernacularName'
-    科 = '$family'
-    属 = '$genus'
-    种加词 = '$specificEpithet'
-    种命名人 = '$specificAuthorship'
-    种下等级 = '$taxonRank'
-    种下加词 = '$infraspecificEpithet'
-    种下等级命名人 = '$scientificNameAuthorship'
-    拉丁名 = BioName('$scientificName', style='scientificName')
-    模式类型 = RadioInput('$typeStatus', 'typeStatus')
-    鉴定人 = HumanName('$identifiedBy')
-    鉴定日期 = DateTime('$dateIdentified')
+    备注 = '$occurrenceRemarks'
     习性 = RadioInput('$habit', 'habit')
     体高 = '$height'
     胸径 = '$DBH'
-    频度 = '$frequency'
-    花 = '$flower'
-    果 = '$fruit'
+    树皮 = '$stem'
     叶 = '$leaf'
-    茎 = '$stem'
-    根 = '$root'
-    种子 = '$seed'
-    野外鉴定 = '$temporaryIdentification'
-    当地名称 = '$dialectName'
-    备注 = '$occurrenceRemarks'
-
+    花 = '$flower'
+    果实 = '$fruit'
+    寄主 = '$host'
+    备注2 = '$organismRemarks'
 
 
 class OccurrenceTerms(Enum):
@@ -88,9 +81,9 @@ class OccurrenceTerms(Enum):
     fieldNotes = '$fieldNotes'
     fundedBy = '$fundedBy'
     # Location
-    country__province__city__county = AdminDiv(('$country', '$province', '$city', '$county', '::'))
+    country__province__city__county = AdminDiv(['$higherGeography', ('$country', '$province', '$city', '$county', '::')])
     locality = ('$locality', '$mountain', '$waterBody', ',')
-    decimalLatitude__decimalLongitude = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
     _minimumElevationInMeters = Number('$minimumElevationInMeters')
     _maximumElevationInMeters = Number('$maximumElevationInMeters')
     minimumElevationInMeters__maximumElevationInMeters = Number('$minimumElevationInMeters', '$maximumElevationInMeters')
@@ -177,7 +170,7 @@ class NoiOccurrenceTerms(Enum):
     county = '$county'
     #country__province__city__county = AdminDiv(('$country', '$province', '$city', '$county', '::'))
     locality = ('$locality', '$mountain', '$waterBody', ',')
-    decimalLatitude__decimalLongitude = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
     _minimumElevationInMeters = Number('$minimumElevationInMeters')
     _maximumElevationInMeters = Number('$maximumElevationInMeters')
     minimumElevationInMeters__maximumElevationInMeters = Number('$minimumElevationInMeters', '$maximumElevationInMeters')
@@ -227,16 +220,19 @@ class KingdoniaPlantTerms(Enum):
     disposition = RadioInput('$disposition', 'disposition')
     preservedLocation = RadioInput('$preservedLocation', 'preservedLocation')
     preservedTime = DateTime('$preservedTime', 'utc')
-    recordedBy = HumanName('$recordedBy')
+    recordedBy = HumanName('$recordedBy', separator=',')
     recordNumber = '$recordNumber'
     eventDate = DateTime('$eventDate', 'datetime')
     _individualCount = Number('$individualCount', None, int)
     individualCount = FillNa('$individualCount', 0)
-    country__stateProvince__city__county = AdminDiv(('$country', '$province', '$city', '$county', '::'))
+    country__stateProvince__city__county = AdminDiv(['$higherGeography', ('$country', '$province', '$city', '$county', '::')])
     locality = ('$locality', '$mountain', '$waterBody', ',')
-    habitat = '$habitat'
+    _habitat = '$habitat'
+    habitat = FillNa('$habitat', '无')
     habit = RadioInput('$habit', 'habit')
-    decimalLatitude__decimalLongitude = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
+    decimalLatitude = FillNa('$decimalLatitude', 0)
+    decimalLongitude = FillNa('$decimalLongitude', 0)
     _minimumElevationInMeters = Number('$minimumElevationInMeters')
     _maximumElevationInMeters = Number('$maximumElevationInMeters')
     minimumElevationInMeters__maximumElevationInMeters = Number('$minimumElevationInMeters', '$maximumElevationInMeters')
@@ -284,25 +280,28 @@ class KingdoniaAvesTerms(Enum):
 
 
 class HerbLabelTerms(Enum):
-    title = '$institutionCode'
-    titleNote = '$collectionCode'
-    subTitle = '$fundedBy'
-    family = '$family'
-    vernacularName = '$vernacularName'
-    genus = '$genus'
-    specificEpithet = '$specificEpithet'
-    specificAuthorship = '$specificAuthorship'
-    taxonRank = '$taxonRank'
-    infraspecificEpithet = '$infraspecificEpithet'
-    scientificNameAuthorship = '$scientificNameAuthorship'
-    scientificName = BioName('$scientificName')
-    identifiedBy = HumanName('$identifiedBy')
-    dateIdentified = DateTime('$dateIdentified', 'date')
-    country__province__city__county = AdminDiv(('$country', '$province', '$city', '$county', '::'))
+    catalogNumber = UniqueID('$catalogNumber')
+    otherCatalogNumbers = '$otherCatalogNumbers'
+    duplicatesOfLabel = Number('$duplicatesOfLabel', None, int)
+    labelTitle = ['$labelTitle', '$institutionCode']
+    labelNote = ['$labelNote', '$collectionCode']
+    labelSubtitle = ['$labelSubtitle', '$fundedBy']
+    recordedBy = HumanName('$recordedBy')
+    recordNumber = '$recordNumber'
+    eventDate = DateTime('$eventDate', 'date')
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
+    minimumElevationInMeters = Number('$minimumElevationInMeters', None, int)
+    maximumElevationInMeters = Number('$maximumElevationInMeters', None, int)
+    country__province__city__county = AdminDiv(['$higherGeography', ('$country', '$province', '$city', '$county', '::')])
     locality = ('$locality', '$mountain', '$waterBody', ',')
     habitat = '$habitat'
     individualCount = Number('$individualCount', None, int)
     habit = RadioInput('$habit', 'habit')
+    family = '$family'
+    vernacularName = '$vernacularName'
+    genus__specificEpithet__taxonRank__infraspecificEpithet__scientificNameAuthorship = BioName([('$genus', '$specificEpithet',  '$specificAuthorship', '$taxonRank', '$infraspecificEpithet', '$scientificNameAuthorship', ' '), '$scientificName'], style='plantSplitName')
+    identifiedBy = HumanName('$identifiedBy')
+    dateIdentified = DateTime('$dateIdentified', 'date')
     flower = '$flower'
     leaf = '$leaf'
     stem = '$stem'
@@ -320,12 +319,6 @@ class HerbLabelTerms(Enum):
     molecularMaterialSample = '$molecularMaterialSample'
     seedMaterialSample = '$seedMaterialSample'
     livingMaterialSample = '$livingMaterialSample'
-    decimalLatitude__decimalLongitude = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
-    minimumElevationInMeters = Number('$minimumElevationInMeters', None, int)
-    maximumElevationInMeters = Number('$maximumElevationInMeters', None, int)
-    recordedBy = HumanName('$recordedBy')
-    recordNumber = '$recordNumber'
-    eventDate = DateTime('$eventDate', 'date')
     occurrenceRemarks = '$occurrenceRemarks'
 
 
@@ -347,9 +340,9 @@ class NsiiTerms(Enum):
     habitat = '$habitat'
     fundedBy = '$fundedBy'
     # Location
-    country__province__city__county = AdminDiv(('$country', '$province', '$city', '$county', '::'))
+    country__province__city__county = AdminDiv(['$higherGeography', ('$country', '$province', '$city', '$county', '::')])
     locality = ('$locality', '$mountain', '$waterBody', ',')
-    decimalLatitude__decimalLongitude = GeoCoordinate(('$decimalLatitude', '$decimalLongitude', ';'))
+    decimalLatitude__decimalLongitude = GeoCoordinate(['$verbatimCoordinates', ('$decimalLatitude', '$decimalLongitude', ';')])
     _minimumElevationInMeters = Number('$minimumElevationInMeters')
     _maximumElevationInMeters = Number('$maximumElevationInMeters')
     minimumElevationInMeters__maximumElevationInMeters = Number('$minimumElevationInMeters', '$maximumElevationInMeters')
